/*
 * Copyright (c) 2019 Meltytech, LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//import QtQuick 2.1
//import QtQuick.Controls 1.0
//import QtQuick.Layouts 1.0

import QtQuick 2.0
import QtQuick.Controls 1.1
import QtQuick.Layouts 1.1
import Shotcut.Controls 1.0
import QtQuick.Dialogs 1.1


Item {
	width: 300
    height: 500
    Component.onCompleted: {
        if (filter.isNew) {
            // Set default parameter values
            filter.set('animation', 0)
            textArea.text = 'Entrez votre texte ici\nsur 1 ou plusieurs lignes'
            filter.set('family', 'arial')
            filter.set('size', 72)
            filter.set('weight', 500)
            filter.set('colorfont', '#ffffff')
            filter.set('border', 0)
            filter.set('colorborder', '#000000')
            filter.set('shadow', 0)
            filter.set('colorshadow', '#000000')
            filter.set('colorbackground', 'purple')
            filter.set('lineheight', 100)
            filter.set('letterspacing', 0)
            filter.set('textalign', 'center')
            filter.set('vertalign', 'middle')
            filter.set('margins', 0)
            filter.set('vitesse_in', 100)
            filter.set('vitesse_out', 100)
        } else {
            // Initialize parameter values
            textArea.text = filter.get('argument')
        }
        animationCombo.currentIndex = filter.getDouble('animation')
        refreshFontButton()
        taille.text = filter.getDouble('size')
        colorFont.value = filter.get('colorfont')
        contour.value = filter.get('border')
        colorBorder.value = filter.get('colorborder')
        shadow.value = filter.get('shadow')
        colorShadow.value = filter.get('colorshadow')
        bgcolor.value = filter.get('colorbackground')
        lineHeight.value = filter.get('lineheight')
        letterspacing.value = filter.get('letterspacing')
        margins.value = filter.get('margins')
        vitesse_in.value = filter.get('vitesse_in')
        vitesse_out.value = filter.get('vitesse_out')
        filter.set('framerate', profile.fps)
        filter.set('duration', filter.duration / profile.fps)
        setRadioButton()
        // Je suppose que le "_" c'est pour ne pas enregistrer ce paramètre dans le .mlt
        filter.set('_reload', 1)
    }
    
    function refreshFontButton() {
        var s = filter.get('family')
        if (filter.getDouble('weight') > 10 * Font.Medium)
            s += ' ' + qsTr('Bold')
        if (filter.get('style') === 'italic')
            s += ' ' + qsTr('Italic')
        fontButton.text = s
    }
    
    function setRadioButton() {
    	var a = filter.get('textalign')
    	if (a == 'left') leftRadioButton.checked = true
    	else if (a == 'right') rightRadioButton.checked = true
    	else centerRadioButton.checked = true
    	var b = filter.get('vertalign')
    	if (b == 'top') topRadioButton.checked = true
    	else if (b == 'bottom') bottomRadioButton.checked = true
    	else middleRadioButton.checked = true
    }
    
    /*function getPosition() {
        return Math.max(producer.position - (filter.in - producer.in), producer.position)
    }*/
    
    function delay(duration) { // In milliseconds
    	var timeStart = new Date().getTime();
    	while (new Date().getTime() - timeStart < duration) {
        	// On ne fait rien, le temps passe
    	}
    	// La tempo est passée, on peut reprendre
	}
	
	function updateFilter() {
		filter.set('_reload', 1)
    }
    
    ExclusiveGroup { id: halignGroup }
    ExclusiveGroup { id: valignGroup }

    GridLayout {
        columns: 3
        anchors.fill: parent
        anchors.margins: 8
        
        Label { 
        	Layout.alignment: Qt.AlignRight
        	text: qsTr('Animation') 
        }

        RowLayout {
            Layout.columnSpan: 2
            
            ComboBox {
                id: animationCombo
                model: [qsTr('Aucune'), qsTr('Fade-in Fade-out pour toute la ligne'),
                qsTr('E : o-->O ; S : o<--O'), qsTr('E :  O<-o->O ; S : O->o<-O'),
                qsTr('E/S Toute la ligne bascule (H)'), qsTr('E/S Toutes la ligne pivote (V)'),
                qsTr('E/S Bascule (H) lettre par lettre'), qsTr('E/S Pivot (V) lettre par lettre'), 
                qsTr('TypeWriter')]
                onCurrentIndexChanged: {
					filter.set('animation', currentIndex)
					updateFilter()
				}
            }
            Item { width: 15 }
            TextArea {
            	id: textArea
            	//Layout.columnSpan: 5
            	textFormat: TextEdit.PlainText
            	wrapMode: TextEdit.NoWrap
            	Layout.minimumHeight: 40
            	Layout.maximumHeight: 55
            	Layout.minimumWidth: 250
            	Layout.maximumWidth: 300
            	text: '__empty__' // workaround initialization problem
            	property int maxLength: 300
            	Keys.onEnterPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('argument', text)
            		updateFilter()
            	}
            	onTextChanged: {
                	if (text === '__empty__') return
                	if (length > maxLength) {
                    	text = text.substring(0, maxLength)
                    	cursorPosition = maxLength
                	}
                	//if (!parseInt(filter.get(useFontSizeProperty)))
                    //	filter.set('size', profile.height / text.split('\n').length)
                	filter.set('argument', text)
            	}
        	}
        }
        
        // ********************** Ligne Police ***************************
        Label { 
        	Layout.alignment: Qt.AlignRight
        	text: qsTr('Police') 
        }
        RowLayout { 
            Layout.columnSpan: 2
            Button {
                id: fontButton
                Layout.preferredWidth: 200
            	onClicked: {
                    fontDialog.open()
                }
                FontDialog {
                    id: fontDialog
                    title: "Choisir une police"
                    modality: Qt.ApplicationModal
                    onFontChanged: {
                        filter.set('family', font.family)
                        filter.set('weight', 10 * font.weight )
                        filter.set('style', font.italic? 'italic' : 'normal' )
                        filter.set('size', font.pixelSize)
                    }
                    onAccepted: {
                    	taille.text = font.pixelSize
                        refreshFontButton()
                        updateFilter()
                    }
                    onRejected: {
                        return
                    }
                }
            }
            Item { width: 15 }
            Label { 
        		Layout.alignment: Qt.AlignRight
        		text: qsTr('Taille ') 
        	}
        	TextField {
        		id: taille
        		Layout.preferredWidth : 40
        		horizontalAlignment: Text.AlignRight
        		text: ''//font.pixelSize
        		Keys.onEnterPressed: focus = false
                Keys.onReturnPressed: focus = false
                onEditingFinished: if (filter.getDouble('size') !== parseFloat(text)) {
                    filter.set('size', parseFloat(text))
                    updateFilter()
                }
        	}
        	Label { 
        		Layout.alignment: Qt.AlignLeft
        		text: qsTr('pt') 
        	}
        	Item { width: 15 }
        	ColorPicker {
                id: colorFont
                alpha: false
                onValueChanged: {
                    filter.set('colorfont', value)
                    updateFilter()
                }
            }
        }
        
        // ********************** Ligne Contour ***************************
        Label { 
        	Layout.alignment: Qt.AlignRight
        	text: qsTr('Contour') 
        }
        RowLayout {
            Layout.columnSpan: 2
            
            Item { width: 25 }
            Label { 
        		Layout.alignment: Qt.AlignRight
        		text: qsTr('Epaisseur ') 
        	}
        	Item { width: 1 }
        	SpinBox {
                id: contour
                Layout.minimumWidth: 40
                minimumValue: 0
                maximumValue: 10
                decimals: 0
                Keys.onEnterPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('border', value)
            		updateFilter()
            	}
                Keys.onReturnPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('border', value)
            		updateFilter()
            	}
                onValueChanged: {
                	if (hovered) forceActiveFocus()
                }
                onEditingFinished: {
                	rafraichir.forceActiveFocus()
                    filter.set('border', value)
                    updateFilter()
                }
            }
            Item { width: 15 }
        	ColorPicker {
                id: colorBorder
                alpha: false
                onValueChanged: {
                    filter.set('colorborder', value)
                    updateFilter()
                }
            }
            // ***** Boutons d'alignement Horizontal *****
            Item { width: 15 }
        	RadioButton {
            	id: leftRadioButton
            	text: 'G'
            	tooltip : 'Aligné à Gauche'
            	exclusiveGroup: halignGroup
            	onClicked:{	
            		filter.set('textalign', 'left')
            		updateFilter()
            	}
        	}
        	Item { width: 15 }
        	RadioButton {
            	id: centerRadioButton
            	text: 'C'
            	tooltip : 'Aligné au Centre'
            	exclusiveGroup: halignGroup
            	onClicked:{	
            		filter.set('textalign', 'center')
            		updateFilter()
            	}
        	}
        	Item { width: 15 }
        	RadioButton {
            	id: rightRadioButton
            	text: 'D'
            	tooltip : 'Aligné à Droite'
            	exclusiveGroup: halignGroup
            	onClicked:{	
            		filter.set('textalign', 'right')
            		updateFilter()
            	}
        	}
        	
        }
        
        // ********************** Ligne Ombrage ***************************
        Label { 
        	Layout.alignment: Qt.AlignRight
        	text: qsTr('Ombrage') 
        }
        RowLayout {
            Layout.columnSpan: 2
            
            Item { width: 25 }
            Label { 
        		Layout.alignment: Qt.AlignRight
        		text: qsTr('Décalage ') 
        	}
        	Item { width: 3 } // pour alignement
        	SpinBox {
                id: shadow
                Layout.minimumWidth: 40
                minimumValue: 0
                maximumValue: 10
                decimals: 0
                Keys.onEnterPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('shadow', value)
            		updateFilter()
            	}
                Keys.onReturnPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('shadow', value)
            		updateFilter()
            	}
                onValueChanged: {
                	if (hovered) forceActiveFocus()
                }
                onEditingFinished: {
                	rafraichir.forceActiveFocus()
                    filter.set('shadow', value)
                    updateFilter()
                }
            }
            Item { width: 15 }
        	ColorPicker {
                id: colorShadow
                alpha: false
                onValueChanged: {
                    filter.set('colorshadow', value)
                    updateFilter()
                }
            }
            // ***** Boutons d'alignement Vertical *****
            Item { width: 15 }
        	RadioButton {
            	id: topRadioButton
            	text: 'H'
            	tooltip : 'Aligné en Haut'
            	exclusiveGroup: valignGroup
            	onClicked:{	
            		filter.set('vertalign', 'top')
            		updateFilter()
            	}
        	}
        	Item { width: 15 }
        	RadioButton {
            	id: middleRadioButton
            	text: 'M'
            	tooltip : 'Aligné au Milieu'
            	exclusiveGroup: valignGroup
            	onClicked:{	
            		filter.set('vertalign', 'middle')
            		updateFilter()
            	}
        	}
        	Item { width: 15 }
        	RadioButton {
            	id: bottomRadioButton
            	text: 'B'
            	tooltip : 'Aligné en Bas'
            	exclusiveGroup: valignGroup
            	onClicked:{	
            		filter.set('vertalign', 'bottom')
            		updateFilter()
            	}
        	}
            
        }
        
        // ********************** Ligne Fond ***************************
        Label { 
        	Layout.alignment: Qt.AlignRight
        	text: qsTr('Fond') 
        }
        RowLayout {
            Layout.columnSpan: 2
            
            Item { width: 10 }
            Button {
                id: transparentButton
                Layout.preferredWidth: 103
                text: qsTr('Transparent')
            	onClicked: {
                    bgcolor.value = 'none'
                }
            }
            Item { width: 3 } // pour alignement
            Label { 
        		Layout.alignment: Qt.AlignRight
        		text: qsTr('Autre ') 
        	}
        	ColorPicker {
                id: bgcolor
                alpha: false
                onValueChanged: {
                    filter.set('colorbackground', value)
                    updateFilter()
                }
            }
            Item { width: 20 }
            Label { 
        		text: qsTr('Marges') 
        	}
        	Item { width: 4 }
            SpinBox {
                id: margins
                Layout.minimumWidth: 72
                minimumValue: 0
                maximumValue: 100
                decimals: 0
                suffix: ' px'
                Keys.onEnterPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('margins', value)
            		updateFilter()
            	}
                Keys.onReturnPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('margins', value)
            		updateFilter()
            	}
                onValueChanged: {
                	if (hovered) forceActiveFocus()
                }
                onEditingFinished: {
                	rafraichir.forceActiveFocus()
                    filter.set('margins', value)
                    updateFilter()
                }
            }
            
        }
        
        // ********************** Ligne Aspect ***************************
        Label { 
        	Layout.alignment: Qt.AlignRight
        	text: qsTr('Aspect') 
        }
        RowLayout {
            Layout.columnSpan: 2
            
            Item { width: 25 }
            Label { 
        		Layout.alignment: Qt.AlignRight
        		text: qsTr('Interligne ') 
        	}
        	Item { width: 3 } // pour alignement
        	SpinBox {
                id: lineHeight
                Layout.minimumWidth: 70
                minimumValue: 50
                maximumValue: 500
                decimals: 0
                suffix: ' %'
                Keys.onEnterPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('lineheight', value)
            		updateFilter()
            	}
                Keys.onReturnPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('lineheight', value)
            		updateFilter()
            	}
                onValueChanged: {
                	if (hovered) forceActiveFocus()
                }
                onEditingFinished: {
                	rafraichir.forceActiveFocus()
                    filter.set('lineheight', value)
                    updateFilter()
                }
            }
            Item { width: 18 }
        	Label { 
        		Layout.alignment: Qt.AlignRight
        		text: qsTr('Lettrage ') 
        	}
        	Item { width: 1 } // pour alignement
        	SpinBox {
                id: letterspacing
                Layout.minimumWidth: 70
                minimumValue: 0
                maximumValue: 10
                decimals: 0
                suffix: ' px'
                Keys.onEnterPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('letterspacing', value)
            		updateFilter()
            	}
                Keys.onReturnPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('letterspacing', value)
            		updateFilter()
            	}
                onValueChanged: {
                	if (hovered) forceActiveFocus()
                }
                onEditingFinished: {
                	rafraichir.forceActiveFocus()
                    filter.set('letterspacing', value)
                    updateFilter()
                }
            }
            Item { width: 3 }
            ToolButton {
            	implicitWidth: 26
            	implicitHeight: 26
            	iconName: 'help-about'
            	iconSource: 'qrc:///icons/oxygen/32x32/actions/help-about.png'
            	tooltip: qsTr('Affiche l\'aide pour ce filtre')
            	onClicked: helpDialog.visible = true
        		MessageDialog {
        			id: helpDialog
        			visible: false
        			modality: Qt.NonModal
        			icon: StandardIcon.Information
        			text: qsTr("<u>Bannière animée</u> : dérivé du programme Elusien<p>"
            		+ "<u>Animation</u> : Choisir une animation parmi les animations disponibles.<p>"
            		+ "<u>Texte</u> : Entrez le texte dans la zone de texte, utilisez la touche 'Retour' pour aller à la ligne.<p>"
                	+ "<u>Marges</u> : Choisir l'alignement du paragraphe sur l'écran (9 possibilités). La position horizontale détermine aussi le centrage du texte dans son bloc. Marges détermine la distance du bloc texte par rapport aux bords de l'écran, cela permet d'éviter d'utiliser un filtre SPR.<p>"
                	+ "<u>Interligne</u> : Règle la distance entre les lignes, laissez 100% pour les textes sur 1 ligne.<p>"
                	+ "<u>Lettrage</u> : Permet d'augmenter la distance entre les lettres.<p>"
                	+ "<u>Vitesse</u> : Anim IN et Anim OUT permettent d'accélérer ou de ralentir les animations d'entrée et de sortie. La valeur par défaut est 100.<p>"
                	+ "<u>Animation</u> : Utilisez les images clés simples pour décaler le départ de l'animation d'entrée et la fin de l'animation de sortie si nécessaire.<p>"
                	+ "D'autres modifications sont possibles en éditant le fichier html qui se trouve dans le même dossier que le fichier .mlt du projet à l'adresse suivante :<p>" + htmlFile.url +"<p>Une fois vos modifications faites et enregistrées, cliquez sur le bouton 'Recharger' pour appliquer la mise à jour.<p>"
                	+ "<u>Bouton OK</u> : Permet de relancer le moteur html pour ré-afficher dans le lecteur (pas fiable à 100%).<p>")
            		standardButtons: StandardButton.Ok
    			}
        	}
            
        }
        
        // ********************** Ligne Vitesse ***************************
        Label { 
        	Layout.alignment: Qt.AlignRight
        	text: qsTr('Vitesse') 
        }
        RowLayout {
            Layout.columnSpan: 2
            
            Item { width: 25 }
            Label { 
        		Layout.alignment: Qt.AlignRight
        		text: qsTr('Anim. IN') 
        	}
        	Item { width: 13 } // pour alignement
        	SpinBox {
                id: vitesse_in
                Layout.minimumWidth: 70
                enabled: animationCombo.currentIndex > 0
                minimumValue: 50
                maximumValue: 200
                decimals: 0
                stepSize: 1
                suffix: ' %'
                Keys.onEnterPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('vitesse_in', value)
            		updateFilter()
            	}
                Keys.onReturnPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('vitesse_in', value)
            		updateFilter()
            	}
                onValueChanged: {
                	if (hovered) forceActiveFocus()
                }
                onEditingFinished: {
                	rafraichir.forceActiveFocus()
                    filter.set('vitesse_in', value)
                    updateFilter()
                }
            }
            Item { width: 7 }
        	Label { 
        		Layout.alignment: Qt.AlignRight
        		text: qsTr('Anim. OUT') 
        	}
        	Item { width: 1 } // pour alignement
        	SpinBox {
                id: vitesse_out
                Layout.minimumWidth: 70
                enabled: animationCombo.currentIndex > 0
                minimumValue: 50
                maximumValue: 200
                decimals: 0
                stepSize: 1
                suffix: ' %'
                Keys.onEnterPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('vitesse_out', value)
            		updateFilter()
            	}
                Keys.onReturnPressed: {
            		rafraichir.forceActiveFocus()
            		filter.set('vitesse_out', value)
            		updateFilter()
            	}
                onValueChanged: {
                	if (hovered) forceActiveFocus()
                }
                onEditingFinished: {
                	rafraichir.forceActiveFocus()
                    filter.set('vitesse_out', value)
                    updateFilter()
                }
            }
            
//* *** Bouton Magouille pour faire réafficher l'image en déplaçant la tête de lecture **********

            Button { // (Bouton OK)
            	id: rafraichir
            	Layout.preferredWidth: 45
				Layout.preferredHeight: 22
				tooltip: qsTr('Cliquez ici pour rafraichir le filtre')
				text: qsTr('OK')
            	onClicked: {
            		forceActiveFocus()
            		delay(500)
            		producer.position = producer.position + 1
        		} 
        	}
        	
// ********** Fin de la magouille ************ */
        }
        	
        
        Item {
            Layout.fillHeight: true
        }
    }
    Connections {
        target: filter
        onInChanged: {
        	filter.set('duration', filter.duration / profile.fps)
        	filter.set('_reload', 1)
        }
        onOutChanged: {
        	filter.set('duration', filter.duration / profile.fps)
        	filter.set('_reload', 1)
        }
    }
}